Attribute VB_Name = "IniFile"
'---------------------------------------------------------------------------
' This program was developed on a Win2K system using Visual Basic 6.0 for
' windows. The program source and binaries are distributed under a BSD
' Style License.
'
' Developed By:
' Matthew T. Linehan
' American LED-gible Inc.
' Phone: 614.851.1100
' email: engineering@ledgible.com
'
' Copyright (C) 2011, American LED-Gible Inc. All rights reserved.
' Redistribution and use in source and binary forms, with or without
' modification, are permitted provided that the following conditions are met:
'
' Redistributions of source code must retain the above copyright notice, this
' list of conditions, and the following disclaimer. Redistributions in binary
' form must reproduce the above copyright notice, this list of conditions and
' the following disclaimer in the documentation and/or other materials provided
' with the distribution.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
' IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
' ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
' LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
' CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
' SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
' INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
' CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
' ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'---------------------------------------------------------------------------

'**************************************************************************************************
'Win32API for Saving a Profile Item to a Private INI file
'
'   lpszSection     - INI File Section String
'   lpszKey         - INI File Key String
'   lpszValue       - INI File Value String
'   lpszFile        - Full Path to the Private INI File to be queried
'
' If the function successfully writes the string to the INI file, the return value is nonzero
'**************************************************************************************************
Private Declare Function WritePrivateProfileString Lib "kernel32" _
   Alias "WritePrivateProfileStringA" ( _
   ByVal lpszSection As String, _
   ByVal lpszKey As String, _
   ByVal lpszValue As String, _
   ByVal lpszFile As String) As Long
   
   
'**************************************************************************************************
'Win32API for Retrieving a Profile Item from a Private INI file
'
'   lpszSection     - INI File Section String
'   lpszKey         - INI File Key String
'   lpszDefault     - INI File Default Value
'   lpszValue       - A pointer to a null-terminated string buffer that will receive the value
'   nSize           - The size of the lpszValue Buffer
'   lpszFile        - Full Path to the Private INI File to be queried
'
' Returns the number of bytes written into the lpszValue Buffer
'**************************************************************************************************
Private Declare Function GetPrivateProfileString Lib "kernel32" _
    Alias "GetPrivateProfileStringA" ( _
    ByVal lpszSection As String, _
    ByVal lpszKey As String, _
    ByVal lpszDefault As String, _
    ByVal lpszValue As String, _
    ByVal nSize As Long, _
    ByVal lpszFile As String) As Long
   
   
'**************************************************************************************************
'Win32API for Retrieving Special Operating System Folders by CSIDL Identifier
'
'   hWndOwner   - Reserved
'   nFolder     - CSIDL Folder Identifier
'   nToken      - An access token that can be used to represent a particular user (Win2K must be NULL)
'   dwFlags     - Flags that specify the path to be returned (Current or Default)
'   lpszPath    - A pointer to a null-terminated string of length MAX_PATH which will receive the path
'
' Returns
'   S_OK            - if successful, request path is written into lpszPath
'   S_FALSE         - The CSIDL in nFolder is valid, but the folder does not exist.
'   E_INVALIDARG    - The CSIDL in nFolder is not valid.
'**************************************************************************************************
Private Declare Function SHGetFolderPath Lib "shell32" _
    Alias "SHGetFolderPathA" ( _
    ByVal hWndOwner As Long, _
    ByVal nFolder As Long, _
    ByVal hToken As Long, _
    ByVal dwFlags As Long, _
    ByVal lpszPath As String) As Long


'**************************************************************************************************
'Constants requrired for SHGetFolderPath()
'nFolder - CSIDL Special Folder Identifiers
'dwFlags - Flag Values
'return  - Return Values
'**************************************************************************************************
'The file system directory that serves as a common repository for application-specific data.
'A typical path is C:\Documents and Settings\username\Application Data
Private Const CSIDL_APPDATA = &H1A

'The file system directory that contains application data for all users.
'A typical path is C:\Documents and Settings\All Users\Application Data
Private Const CSIDL_COMMON_APPDATA = &H23

'The file system directory that contains files and folders that appear on the desktop for all users.
'A typical path is C:\Documents and Settings\All Users\Desktop
Private Const CSIDL_COMMON_DESKTOPDIRECTORY = &H19

'The file system directory that contains documents that are common to all users.
'A typical paths is C:\Documents and Settings\All Users\Documents
Private Const CSIDL_COMMON_DOCUMENTS = &H2E

'The file system directory that contains the common program groups that appear on the Start menu for all users.
'A typical path is C:\Documents and Settings\All Users\Start Menu\Programs
Private Const CSIDL_COMMON_PROGRAMS = &H17

'The file system directory that contains the programs and folders that appear on the Start menu for all users.
'A typical path is C:\Documents and Settings\All Users\Start Menu
Private Const CSIDL_COMMON_STARTMENU = &H16

'The file system directory that contains the programs that appear in the Startup folder for all users.
'A typical path is C:\Documents and Settings\All Users\Start Menu\Programs\Startup
Private Const CSIDL_COMMON_STARTUP = &H18

'The file system directory used to physically store file objects on the desktop.
'A typical path is C:\Documents and Settings\username\Desktop
Private Const CSIDL_DESKTOPDIRECTORY = &H10

'The file system directory that serves as a data repository for local (nonroaming) applications.
'A typical path is C:\Documents and Settings\username\Local Settings\Application Data
Private Const CSIDL_LOCAL_APPDATA = &H1C

'The file system directory that serves as a common repository for image files.
'A typical path is C:\Documents and Settings\username\My Documents\My Pictures
Private Const CSIDL_MYPICTURES = &H27

'The user's profile folder.
'A typical path is C:\Users\username
Private Const CSIDL_PROFILE = &H28

'The Program Files folder.
'A typical path is C:\Program Files
Private Const CSIDL_PROGRAM_FILES = &H26

'A folder for components that are shared across applications.
'A typical path is C:\Program Files\Common
Private Const CSIDL_PROGRAM_FILES_COMMON = &H2B

'The file system directory that contains the user's program groups.
'A typical path is C:\Documents and Settings\username\Start Menu\Programs
Private Const CSIDL_PROGRAMS = &H2

'The file system directory that contains Start menu items.
'A typical path is C:\Documents and Settings\username\Start Menu
Private Const CSIDL_STARTMENU = &HB

'The file system directory that corresponds to the user's Startup program group.
'A typical path is C:\Documents and Settings\username\Start Menu\Programs\Startup
Private Const CSIDL_STARTUP = &H7

'The Windows System folder.
'A typical path is C:\Windows\System32
Private Const CSIDL_SYSTEM = &H25

'The Windows directory or SYSROOT. This corresponds to the %windir% or %SYSTEMROOT% environment variables.
'A typical path is C:\Windows
Private Const CSIDL_WINDOWS = &H24

'Combine with another CSIDL to force the creation of the associated folder if it does not exist
Private Const CSIDL_FLAG_CREATE = &H8000&

'Retrieve the folder's current path
Private Const SHGFP_TYPE_CURRENT = &H0

'Retrieve the folder's default path
Private Const SHGFP_TYPE_DEFAULT = &H1

'Requested Path is stored at lpzPath
Private Const S_OK = 0

'The CSIDL  in nFolder is valid, but the folder does not exist
Private Const S_FALSE = 1

'The CSIDL  in nFolder is not valid
Private Const E_INVALIDARG = &H80070057

'Win32 Maximim Path Length Limit
Private Const MAX_PATH = 260


'**************************************************************************************************
'A thin VB6 Wrapper for the Win32 SHGetFolderPath() API in Shell32.dll
'**************************************************************************************************
Public Function VbGetFolderPath(ByVal nFolder As Long, ByVal dwFlags As Long) As String
    Dim strPath As String
    Dim lngResult As Long
    
    'The OS, and VB6 do not aggree on the defintion of a string
    'to work around this, we have to force VB6 to allocate a
    'string buffer large enough to store the largest possible result
    strPath = Space$(MAX_PATH)
    
    'Ask the OS for the CSIDL specified path
    lngResult = SHGetFolderPath(0&, nFolder, 0&, dwFlags, strPath)
    
    'Parce the result of the call into SHGetFolderPath()
    Select Case lngResult
    
        Case S_OK           'Return Specified Path String to Caller
            VbGetFolderPath = Left$(strPath, InStr(strPath, vbNullChar) - 1)
            
        Case S_FALSE        'Raise an error
            Err.Raise vbObjectError + &HB112&, _
                "VbGetFolderPath()", _
                "Valid PathId but folder doesn't exist"
                      
        Case E_INVALIDARG   'Raise an error
            Err.Raise vbObjectError + &HB114&, _
                "VbGetFolderPath()", _
                "Invalid PathId for requested function"
                      
        Case Else           'Raise an Error
            Err.Raise vbObjectError + &HB116&, _
                "VbGetFolderPath()", _
                "System error " & CStr(lngResult)
                      
    End Select
End Function


'**************************************************************************************************
'Function to detect if the program is running inside the IDE, or as a stand alone program
'returns TRUE if the program is running within the VB6 Integrated Development Environment
'**************************************************************************************************
Public Function InIDE() As Boolean
    On Error Resume Next    'Do not terminate on an error
    Debug.Assert 1 / 0      'If we are running under the IDE, this generates a divide by zero error
    InIDE = (Err <> 0)      'Reduce the Error Result to a Boolean True / False
End Function


'**************************************************************************************************
'A function to retrieve the directory where common application data is stored
'**************************************************************************************************
Public Function GetCommonAppDataFolder() As String
    
    'If we are running in the Integrated Development Environment
    If (InIDE() = True) Then
        'then returnt the Source Code Directory instead of the Real Common AppData Directory
        GetCommonAppDataFolder = App.Path
    Else
        'else return the Common Application Data Diretory (Varies by OS Version)
        GetCommonAppDataFolder = VbGetFolderPath(CSIDL_COMMON_APPDATA, SHGFP_TYPE_CURRENT)
        GetCommonAppDataFolder = GetCommonAppDataFolder & "\" & App.Title
    End If

End Function


'**************************************************************************************************
'A function to retrieve the directory where common documents are stored
'**************************************************************************************************
Public Function GetCommonDocumentsFolder() As String
    
    'If we are running in the Integrated Development Environment
    If (InIDE() = True) Then
        'then return the Source Code Directory instead of the Real Common Documents Directory
        GetCommonDocumentsFolder = App.Path
    Else
        'else return the Common Documents Directory (Varies by OS Version)
        GetCommonDocumentsFolder = VbGetFolderPath(CSIDL_COMMON_DOCUMENTS, SHGFP_TYPE_CURRENT)
        GetCommonDocumentsFolder = GetCommonDocumentsFolder & "\" & App.Title
        
    End If

End Function


'**************************************************************************************************
'A simple function to write a profile string to a private INI file
'**************************************************************************************************
Public Sub WriteIni(File As String, Section As String, Key As String, Value As String)
   Call WritePrivateProfileString(Section, Key, Value, File)
End Sub
   

'**************************************************************************************************
'A simple function to read a profile string from a private INI file
'**************************************************************************************************
Public Function ReadIni(File As String, Section As String, Key As String, Default As String) As String
    Dim Value As String
    Dim Size As Long
    
    'The OS, and VB6 do not aggree on the defintion of a string
    'to work around this, we have to force VB6 to allocate a
    'string buffer large enough to store the largest possible result
    Value = Space$(255)
    Size = Len(Value)
    
    'Ask the OS to read the INI file for us
    Size = GetPrivateProfileString(Section, Key, Default, Value, Size, File)
    
    'Parce the result of the GetPrivateProfileString OS call
    If (Size > 0) Then
        ReadIni = Mid(Value, 1, Size)
    Else
        ReadIni = ""
    End If
End Function


'**************************************************************************************************
'Make a Directory Path
'**************************************************************************************************
Public Sub MakePath(ByVal Path As String)
Dim directories As Variant
Dim i As Integer
Dim new_dir As String
Dim dir_path As String

    'Break the path into componet directories.
    directories = Split(Path, "\")

    'Walk the path, left to right, one directory at a time
    For i = LBound(directories) To UBound(directories)
    
        'Get the next directory in the path.
        new_dir = directories(i)

        'If the next directory is not zero length
        If Len(new_dir) > 0 Then
        
            'Append it to the working path
            dir_path = dir_path & new_dir & "\"

            'Make sure we don't just have a drive specification.
            If Right$(new_dir, 1) <> ":" Then
            
                'If the path does not exist
                If Dir$(dir_path, vbDirectory) = "" Then
                
                    'Make it.
                    MkDir dir_path
                    
                End If
                
            End If
            
        End If
        
    Next i
    
End Sub


'**************************************************************************************************
'A Simple ROT13 to Scramble Passwords
'**************************************************************************************************
Public Function ROT13(SecretWord As String) As String
    Dim i As Integer
    Dim c As Integer

    For i = 1 To Len(SecretWord)
        c = Asc(Mid(SecretWord, i, 1))
        If (c >= Asc("A") And c <= Asc("Z")) Then
            c = c + 13
            If (c > Asc("Z")) Then c = c - 26
        ElseIf (c >= Asc("a") And c <= Asc("z")) Then
            c = c + 13
            If (c > Asc("z")) Then c = c - 26
        End If
        ROT13 = ROT13 & Chr(c)
    Next i

End Function

