Attribute VB_Name = "Module1"
'---------------------------------------------------------------------------
' This program was developed on a Win2K system using Visual Basic 6.0 for
' windows. The program source and binaries are distributed under a BSD
' Style License.
'
' Developed By:
' Matthew T. Linehan
' American LED-gible Inc.
' Phone: 614.851.1100
' email: engineering@ledgible.com
'
' Copyright (C) 2011, American LED-Gible Inc. All rights reserved.
' Redistribution and use in source and binary forms, with or without
' modification, are permitted provided that the following conditions are met:
'
' Redistributions of source code must retain the above copyright notice, this
' list of conditions, and the following disclaimer. Redistributions in binary
' form must reproduce the above copyright notice, this list of conditions and
' the following disclaimer in the documentation and/or other materials provided
' with the distribution.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
' IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
' ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
' LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
' CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
' SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
' INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
' CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
' ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'---------------------------------------------------------------------------
' V2.0 01-09-2012 Rewrote Trimline Monitor from Scratch,
'                 Using PPT Monitor 4.0 as a starting point (PD-2116-051)
' V2.1 01-19-2010 Moved Data Store from Registry to Common Documents Folder
'                 Win7 = c:\users\public\documents
'                 XP   = c:\documents and settings\all users\documents
'                 Win98= c:\My Documents
'---------------------------------------------------------------------------
Option Explicit

'---------------------------------------------------------------------------
' Declare Data Structure/Storage for the Trimline Counters / Timers in RAM
'---------------------------------------------------------------------------
Public Const NUMTLU As Integer = 64

'Declare structure for tracking a TLU Register in RAM
Type TLUREG
    Value As Long               'Register Value (Read)
    Status As String            'Register Status
End Type

'Declare structure for tracking a single Trimline Unit in RAM
Type TLUTYPE
    Name As String              'Trimline Symbolic Name
    Port As Integer             'Comport 1 to 16 (0=Disabled, 1=COM1:, 16=TCP)
    Address As Integer          'Address character 32 to 126
    HostName As String          'TCP/IP Host Name
    Schedule As Integer         'Schedule Selection 1 to 64
    Actual As TLUREG            'Actual Count/Time Register
    Status As String            'Trimline Status
    LastCheck As Date           'Last Time Schedule was Checked
    CmdQueue As Queue           'Pending Commands for this Trimline
End Type

'Dim an array to store all data for all Trimlines's in RAM
Public Tlu(1 To NUMTLU) As TLUTYPE


'---------------------------------------------------------------------------
' Declare Data Structure/Storage for the Trimline Counters / Timers on Disk
'---------------------------------------------------------------------------
'Declare structure for tracking a TLU Register on Disk
Type TLUREGDISK
    Value As Long               'Register Value (Read)
    Status As String * 256      'Register Status
End Type

'Declare structure for tracking a single Trimline Unit on Disk
Type TLUTYPEDISK
    Name As String * 256        'Trimline Symbolic Name
    Port As Integer             'Comport 1 to 16 (0=Disabled, 1=COM1:, 16=TCP)
    Address As Integer          'Address character 32 to 126
    HostName As String * 256    'TCP/IP Host Name
    Schedule As Integer         'Schedule Selection 1 to 64
    Actual As TLUREGDISK        'Actual Count/Time Register
    Status As String * 256      'Trimline Status
    LastCheck As Date           'Last Time Schedule was Checked
End Type


'---------------------------------------------------------------------------
' Declare Data Structure/Storage for the Slave Totalizer Displays in RAM
'---------------------------------------------------------------------------
Public Const NUMTOTALIZER As Integer = 64

'Declare a structure for storing a Totalizer State in RAM
Type TOTALIZERTYPE
    Name As String                          'Symbolic Name of this Totalizer
    Port As Integer                         'Comport 1 to 16 (0=Disabled, 1=COM1:, 16=TCP))
    HostName As String                      'TCP/IP HostName
    ActualAddress As Integer                'Address Character 32 to 126
    TluIsMember(1 To NUMTLU) As Integer     'Is this Trimline Unit used in the Calculation
    TotalActual As Long                     'Result Total Actual
    Status As String                        'Communications Status
End Type

'Global Storrage for All Named Totalizer Displays in RAM
Public Totalizer(1 To NUMTOTALIZER) As TOTALIZERTYPE


'---------------------------------------------------------------------------
' Declare Data Structure/Storage for the Slave Totalizer Displays on Disk
'---------------------------------------------------------------------------
'Declare a structure for storing a Totalizer State on Disk
Type TOTALIZERTYPEDISK
    Name As String * 256                    'Symbolic Name of this Totalizer
    Port As Integer                         'Comport 1 to 16 (0=Disabled, 1=COM1:, 16=TCP))
    HostName As String * 256                'TCP/IP HostName
    ActualAddress As Integer                'Address Character 32 to 126
    TluIsMember(1 To NUMTLU) As Integer     'Is this Trimline Unit used in the Calculation
    TotalActual As Long                     'Result Total Actual
    Status As String * 256                  'Communications Status
End Type


'---------------------------------------------------------------------------
' Declare Data Structure/Storage for the Named Schedules in RAM
'---------------------------------------------------------------------------
Public Const NUMSCHEDULE As Integer = 64
Public Const NUMSHIFT As Integer = 8

'Declare a structure for storing a Scheduled Shift in RAM
Type SHIFTTYPE
    Rst As Integer      'Rest at start of shift?
    Exp As Integer      'CSV Export at end of shift?
    Sun As Integer      'Enabled on Sunday
    Mon As Integer      'Enabled on Monday
    Tue As Integer      'Enabled on Tuesday
    Wed As Integer      'Enabled on Wednesday
    Thr As Integer      'Enabled on Thursday
    Fri As Integer      'Enabled on Friday
    Sat As Integer      'Enabled on Saturday
    Beg As Date         'Begin Time
    End As Date         'End Time
End Type

'Declare a structure for storing a Named Schedule in RAM
Type SCHEDULETYPE
    Name As String                      'Schedule Symbolic Name
    Shift(1 To NUMSHIFT) As SHIFTTYPE   '8 Shifts per Named Schedule
End Type

'Global Storage for All Named Schedules in RAM
Public Schedule(1 To NUMSCHEDULE) As SCHEDULETYPE


'---------------------------------------------------------------------------
' Declare Data Structure/Storage for the Named Schedules on Disk
'---------------------------------------------------------------------------
'Declare a structure for storing a Scheduled Shift
Type SHIFTTYPEDISK
    Rst As Integer      'Rest at start of shift?
    Exp As Integer      'CSV Export at end of shift?
    Sun As Integer      'Enabled on Sunday
    Mon As Integer      'Enabled on Monday
    Tue As Integer      'Enabled on Tuesday
    Wed As Integer      'Enabled on Wednesday
    Thr As Integer      'Enabled on Thursday
    Fri As Integer      'Enabled on Friday
    Sat As Integer      'Enabled on Saturday
    Beg As Date         'Begin Time
    End As Date         'End Time
End Type

'Declare a structure for storing a Named Schedule on Disk
Type SCHEDULETYPEDISK
    Name As String * 256                  'Schedule Symbolic Name
    Shift(1 To NUMSHIFT) As SHIFTTYPEDISK '8 Shifts per Named Schedule
End Type


'---------------------------------------------------------------------------
' Misc Global Variables for Internal State
'---------------------------------------------------------------------------
'Storage for misc program state
Public OpState As Boolean       'Make false to kill background functions
Public SecretWord As String     'Password to make changes
Public DebugFlag As Boolean     'Is Debuging Enabled?

'Queue for Program to User Messages
Public MsgQueue As New Queue

'---------------------------------------------------------------------------
' Declare Windows API Functions we Need...
'---------------------------------------------------------------------------
Private Declare Sub Sleep Lib "kernel32.dll" (ByVal dwMilliseconds As Long)


'###########################################################################
' MAIN()
' The whole program begins and ends here!
'###########################################################################
Public Sub Main()
    'Create the main form and show it
    frmMain.Show
    Call YieldProcessor
    
    'Load the System Configuration
    Call LoadAllConfiguration
    
    'Debug Log Start
    Call LogEvent("Main()", "Program Start")
    
    'Start the background comunications task
    OpState = True              'set to false to kill the background task
    Call BackgroundFunction     'This will not return until OpState is false
    
    'Debug Log End
    Call LogEvent("Main()", "Program End")
    
    'Save the system configuration before we end
    Call SaveAllConfiguration
    
    'Make sure the program terminates
    End

End Sub


'###########################################################################
' Background Trimline Unit Communications
' Background Totalizer Communications
' Background Schedule Processing
'###########################################################################
Public Sub BackgroundFunction()
    Dim Unit As Integer
    On Error Resume Next
    
    'This function runs in the "Background" and
    'handles all comunications with the Trimlines's
    'This sub is put in the background by carful use of DoEvents
    'Set OpState = False to kill this sub, or it loops forever
    
    'Load the Winsock Controls
    For Unit = LBound(Tlu) To UBound(Tlu)
        Load frmMain.Winsock1(Unit)
    Next Unit
    For Unit = LBound(Totalizer) To UBound(Totalizer)
        Load frmMain.Winsock2(Unit)
    Next Unit
    
    'For as long as the programm is running
    While OpState
    
        'Communicate with all possible Trimline units
        For Unit = LBound(Tlu) To UBound(Tlu)
            
            ' Yield to the main form for a bit
            Call YieldProcessor
            
            'Check to see if were still running, if not exit loop now
            If Not OpState Then Exit For
            
            'Try to Communicate with the Trimline Unit
            Call CommunicateWithTlu(Unit)
            
        Next Unit
        
        'Communicate with all possible Totalizer units
        For Unit = LBound(Totalizer) To UBound(Totalizer)
            
            ' Yield to the main form for a bit
            Call YieldProcessor
            
            'Check to see if were still running, if not exit loop now
            If Not OpState Then Exit For
            
            'Try to Communicate with the Totalizer
            Call CommunicateWithTotalizer(Unit)
            
        Next Unit
        
    Wend 'loop for as long as the program is running
    
    'Make sure the serial ports and winsocks are closed before exiting
    If (frmMain.MSComm1.PortOpen = True) Then frmMain.MSComm1.PortOpen = False
    For Unit = LBound(Tlu) To UBound(Tlu)
        If frmMain.Winsock1(Unit).State <> sckClosed Then
            Call frmMain.Winsock1(Unit).Close
        End If
    Next Unit
    For Unit = LBound(Totalizer) To UBound(Totalizer)
        If frmMain.Winsock2(Unit).State <> sckClosed Then
            Call frmMain.Winsock2(Unit).Close
        End If
    Next Unit
    
End Sub


'---------------------------------------------------------------------------
' Attempt to Communicate with the Specified Trimline Unit
'---------------------------------------------------------------------------
Public Sub CommunicateWithTlu(Unit As Integer)
    On Error Resume Next
    
    'If the Trimline is SERIAL
    If (Tlu(Unit).Port >= 1 And Tlu(Unit).Port <= 15) Then
        
        'Make sure the Associated Winsock Control is closed
        If frmMain.Winsock1(Unit).State <> sckClosed Then
            Call frmMain.Winsock1(Unit).Close
        End If
        
        'Communicate With Trimline Via Comport
        Call CommunicateWithTluViaCom(Unit)
        
    'If the Trimline is TCP/IP
    ElseIf (Tlu(Unit).Port = 16) Then
    
        'Communicate With the Trimline Via TCP/IP
        Call CommunicateWithTluViaTcp(Unit)
        
    'Else the Trimline is UNUSED / UNKNOWN
    Else
    
        'Make sure the Associated Winsock Control is closed
        If frmMain.Winsock1(Unit).State <> sckClosed Then
            Call frmMain.Winsock1(Unit).Close
        End If
        
        'Make Sure Trimline State is UNUSED
        Call EraseTluConfiguration(Unit)
        Call UpdateTluForms(Unit)
        Call YieldProcessor
    
    End If
    
End Sub


'---------------------------------------------------------------------------
' Attempt to Communicate with the Specified Trimline Via Serial Port
'---------------------------------------------------------------------------
Public Sub CommunicateWithTluViaCom(Unit As Integer)

    'If the currently select COM port is wrong for this Trimline
    If frmMain.MSComm1.CommPort <> Tlu(Unit).Port Then  'If the wrong COM is selected
        If (frmMain.MSComm1.PortOpen = True) Then       'And the COM control is Open
            frmMain.MSComm1.PortOpen = False            'close it to allow reprogramming
        End If
    End If
    
    'If the COM control is closed, try to open it
    If frmMain.MSComm1.PortOpen = False Then        'If the Com Control is Closed
        frmMain.MSComm1.CommPort = Tlu(Unit).Port   'Use the Specified Port
        frmMain.MSComm1.Handshaking = comNone       'With all handshacking disabled
        frmMain.MSComm1.Settings = "19200,n,8,1"    'at 19200 baud, no parity, 8 data, 1 stop
        frmMain.MSComm1.PortOpen = True             'Try to open the com control
    End If

    'If the Com Control is open, Exchange data with the Trimline Unit
    If frmMain.MSComm1.PortOpen = True Then
    
        If (OpState) Then Call CheckTluSchedule(Unit)
        If (OpState) Then Call WriteTluRegsViaCom(Unit)
        If (OpState) Then Call ReadTluRegsViaCom(Unit)
        Call SaveTluConfiguration(Unit)
        Call YieldProcessor
    
    Else
        
        'Comport failed to open
        'flag PortError on all registers for this Trimline Unit
        Call SetTluStatus(Unit, "COM Port Error")
        Call UpdateTluForms(Unit)
    
    End If
    
End Sub


'---------------------------------------------------------------------------
' Attempt to Communicate with the Specified Trimline via TCP/IP
'---------------------------------------------------------------------------
Public Sub CommunicateWithTluViaTcp(Unit As Integer)

    'TCP Connection State Machine...
    Select Case (frmMain.Winsock1(Unit).State)
    
        'TCP Socket is Closed
        Case sckClosed
            Call SetTluStatus(Unit, "TCP Closed")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'Open a Connection to the Trimline Unit
            frmMain.Winsock1(Unit).Protocol = sckTCPProtocol
            frmMain.Winsock1(Unit).RemoteHost = Tlu(Unit).HostName
            frmMain.Winsock1(Unit).RemotePort = 3001
            frmMain.Winsock1(Unit).Connect
            
        'TCP Socket is Open
        Case sckOpen
            Call SetTluStatus(Unit, "TCP Open")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'Invalid State, Close Socket
            Call frmMain.Winsock1(Unit).Close
            
        'TCP Socket is Listening for an Inbound Connection Request
        Case sckListening
            Call SetTluStatus(Unit, "TCP Listening...")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'Invalid State, close Socket
            Call frmMain.Winsock1(Unit).Close
            
        'TCP Socket Inbound Connection Request is Pending
        Case sckConnectionPending
            Call SetTluStatus(Unit, "Connection Pending...")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'Invalid State, Close Slocket
            Call frmMain.Winsock1(Unit).Close
            
        'TCP Socket is Performing a DNS Lookup
        Case sckResolvingHost
            Call SetTluStatus(Unit, "Resolving Host...")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'Waiting on DNS to Resolve HostName...
            'Will Auto Advance to sckHostResolved or sckError
            
        'TCP Socket DNS Lookup Completed
        Case sckHostResolved
            Call SetTluStatus(Unit, "TCP Host Resolved")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'DNS Resolved HostName
            'Will Auto Advance to Connecting...
            
        'TCP Socket is Initiating an Outbound Connection Request
        Case sckConnecting
            Call SetTluStatus(Unit, "TCP Connecting...")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'Waiting on 3 Way ACK from Trimline...
            'Will Auto Advance to sckConnected or sckError
            
        'TCP Socket is Connected to a Remote Host
        Case sckConnected
            'Call SetTluStatus(Unit, "TCP Connected")
            'Call UpdateTluForms(Unit)
            'Call YieldProcessor
            
            'Connected to a Trimline, Exchange Data or Disconnect
            If (frmMain.Winsock1(Unit).RemoteHost = Tlu(Unit).HostName) Then
                If (OpState) Then Call CheckTluSchedule(Unit)
                If (OpState) Then Call WriteTluRegsViaTcp(Unit)
                If (OpState) Then Call ReadTluRegsViaTcp(Unit)
                Call SaveTluConfiguration(Unit)
                Call YieldProcessor
            Else
                Call frmMain.Winsock1(Unit).Close
            End If
            
        'TCP Socket is Closing due to Remote Shutdown Request
        Case sckClosing
            Call SetTluStatus(Unit, "TCP Closing...")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'Remote Host Wants to Close the Connection
            Call frmMain.Winsock1(Unit).Close
            
        'An Error has Occured
        Case sckError
            Call SetTluStatus(Unit, "TCP Error")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'Invalid State, Close the Socket
            Call frmMain.Winsock1(Unit).Close
            
        'The TCP Socket is in an Unknown State
        Case Else
            Call SetTluStatus(Unit, "TCP Unknown State")
            Call UpdateTluForms(Unit)
            Call YieldProcessor
            
            'Invalid State, Close the Connection
            Call frmMain.Winsock1(Unit).Close
            
    End Select
    
End Sub


'---------------------------------------------------------------------------
' Attempt to Communicate with the Specified Totalizer
'---------------------------------------------------------------------------
Public Sub CommunicateWithTotalizer(Unit As Integer)
    On Error Resume Next
    
    'If the Totalizer is SERIAL
    If (Totalizer(Unit).Port >= 1 And Totalizer(Unit).Port <= 15) Then
        
        'Make sure the Associated Winsock Control is closed
        If frmMain.Winsock2(Unit).State <> sckClosed Then
            Call frmMain.Winsock2(Unit).Close
        End If
        
        'Communicate With Totalizer Via Comport
        Call ComputeTotalizerTotals(Unit)
        Call CommunicateWithTotalizerViaCom(Unit)
        Call UpdateTotalizerForms(Unit)
        
    'If the Totalizer is TCP/IP
    ElseIf (Totalizer(Unit).Port = 16) Then
    
        'Communicate With the Totalizer Via TCP/IP
        Call ComputeTotalizerTotals(Unit)
        Call CommunicateWithTotalizerViaTcp(Unit)
        Call UpdateTotalizerForms(Unit)
        
    'Else the Totalizer is UNUSED / UNKNOWN
    Else
    
        'Make sure the Associated Winsock Control is closed
        If frmMain.Winsock2(Unit).State <> sckClosed Then
            Call frmMain.Winsock2(Unit).Close
        End If
        
        'Make Sure Totalizer State is UNUSED
        Call EraseTotalizerConfiguration(Unit)
        Call UpdateTotalizerForms(Unit)
        Call YieldProcessor
    
    End If
    
End Sub


'---------------------------------------------------------------------------
' Compute New Totals for the Specified Totalizer
'---------------------------------------------------------------------------
Public Sub ComputeTotalizerTotals(Unit As Integer)
    Dim TotalActual As Long
    Dim i As Integer
    
    'Iterate over all Trimline Totalizer Members of Totalizer Group
    Totalizer(Unit).Status = "Calculating..."
    Call UpdateTotalizerForms(Unit)
    Call YieldProcessor
    For i = LBound(Totalizer(Unit).TluIsMember) To UBound(Totalizer(Unit).TluIsMember)
        If (Totalizer(Unit).TluIsMember(i) <> 0) Then
            TotalActual = TotalActual + Tlu(i).Actual.Value
        End If
    Next i
    
    'Update Totalizer Totals
    Totalizer(Unit).TotalActual = TotalActual
    Totalizer(Unit).Status = "Calculation Done"
    Call UpdateTotalizerForms(Unit)
    Call YieldProcessor
    
End Sub


'---------------------------------------------------------------------------
' Attempt to Communicate with the Specified Totalizer Via Serial Port
'---------------------------------------------------------------------------
Public Sub CommunicateWithTotalizerViaCom(Unit As Integer)

    'If the currently select COM port is wrong for this Totalizer
    If frmMain.MSComm1.CommPort <> Totalizer(Unit).Port Then    'If the wrong COM is selected
        If (frmMain.MSComm1.PortOpen = True) Then               'And the COM control is Open
            frmMain.MSComm1.PortOpen = False                    'close it to allow reprogramming
        End If
    End If
    
    'If the COM control is closed, try to open it
    If frmMain.MSComm1.PortOpen = False Then                'If the Com Control is Closed
        frmMain.MSComm1.CommPort = Totalizer(Unit).Port     'Use the Specified Port
        frmMain.MSComm1.Handshaking = comNone               'With all handshacking disabled
        frmMain.MSComm1.Settings = "19200,n,8,1"            'at 19200 baud, no parity, 8 data, 1 stop
        frmMain.MSComm1.PortOpen = True                     'Try to open the com control
    End If

    'If the Com Control is open, Send Data to the Totalizer
    If frmMain.MSComm1.PortOpen = True Then
    
        If (OpState) Then Call WriteTotalizerViaCom(Unit)
        Call YieldProcessor
    
    Else
        
        'Comport failed to open
        'flag PortError on all registers for this Trimline
        Totalizer(Unit).Status = "COM Port Error"
        Call UpdateTotalizerForms(Unit)
    
    End If

End Sub


'---------------------------------------------------------------------------
' Write Totalizer Data to Totalizer via Serial Port
'---------------------------------------------------------------------------
Public Sub WriteTotalizerViaCom(Unit As Integer)
    Dim STX As String
    Dim ActAddr As String
    Dim ETX As String
    Dim TXBuf As String
    Dim RXBuf As String
    
    With Totalizer(Unit)
    
        'Define strings to make the program more readable
        STX = Chr$(2)                       'CTRL-B, Start of transmition
        ActAddr = Chr(.ActualAddress)       'Field Address Character
        ETX = Chr$(3)                       'CTRL-C, End of transmition
    
        'Tell the world were working on this Totalizer
        .Status = "Writing..."
        Call UpdateTotalizerForms(Unit)
    
        'Flush the Receive Buffer
        While frmMain.MSComm1.InBufferCount
            RXBuf = frmMain.MSComm1.Input
        Wend
        RXBuf = ""
     
        'Transmit the Write Command to the Totalizer
        TXBuf = ""
        If (.ActualAddress <> 0) Then TXBuf = TXBuf & STX & ActAddr & fmtNumber(.TotalActual) & ETX
        frmMain.MSComm1.Output = TXBuf
        
    End With
    
    'Tell the world were working on this Totalizer
    Totalizer(Unit).Status = "Done"
    
End Sub


'---------------------------------------------------------------------------
' Attempt to Communicate with the Specified Totalizer via TCP/IP
'---------------------------------------------------------------------------
Public Sub CommunicateWithTotalizerViaTcp(Unit As Integer)

    'TCP Connection State Machine...
    Select Case (frmMain.Winsock2(Unit).State)
    
        'TCP Socket is Closed
        Case sckClosed
            Totalizer(Unit).Status = "TCP Closed"
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'Open a Connection to the Totalizer
            frmMain.Winsock2(Unit).Protocol = sckTCPProtocol
            frmMain.Winsock2(Unit).RemoteHost = Totalizer(Unit).HostName
            frmMain.Winsock2(Unit).RemotePort = 3001
            frmMain.Winsock2(Unit).Connect
            
        'TCP Socket is Open
        Case sckOpen
            Totalizer(Unit).Status = "TCP Open"
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'Invalid State, Close Socket
            Call frmMain.Winsock2(Unit).Close
            
        'TCP Socket is Listening for an Inbound Connection Request
        Case sckListening
            Totalizer(Unit).Status = "TCP Listening..."
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'Invalid State, close Socket
            Call frmMain.Winsock2(Unit).Close
            
        'TCP Socket Inbound Connection Request is Pending
        Case sckConnectionPending
            Totalizer(Unit).Status = "Connection Pending..."
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'Invalid State, Close Slocket
            Call frmMain.Winsock2(Unit).Close
            
        'TCP Socket is Performing a DNS Lookup
        Case sckResolvingHost
            Totalizer(Unit).Status = "Resolving Host..."
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'Waiting on DNS to Resolve HostName...
            'Will Auto Advance to sckHostResolved or sckError
            
        'TCP Socket DNS Lookup Completed
        Case sckHostResolved
            Totalizer(Unit).Status = "TCP Host Resolved"
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'DNS Resolved HostName
            'Will Auto Advance to Connecting...
            
        'TCP Socket is Initiating an Outbound Connection Request
        Case sckConnecting
            Totalizer(Unit).Status = "TCP Connecting..."
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'Waiting on 3 Way ACK from Totalizer...
            'Will Auto Advance to sckConnected or sckError
            
        'TCP Socket is Connected to a Remote Host
        Case sckConnected
            'Totalizer(Unit).Status = "TCP Connected"
            'Call UpdateTotalizerForms(Unit)
            'Call YieldProcessor
            
            'Connected to a Totalizer, Write Data or Disconnect
            If (frmMain.Winsock2(Unit).RemoteHost = Totalizer(Unit).HostName) Then
                If (OpState) Then Call WriteTotalizerViaTcp(Unit)
                Call SaveTotalizerConfiguration(Unit)
                Call YieldProcessor
            Else
                Call frmMain.Winsock2(Unit).Close
            End If
            
        'TCP Socket is Closing due to Remote Shutdown Request
        Case sckClosing
            Totalizer(Unit).Status = "TCP Closing..."
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'Remote Host Wants to Close the Connection
            Call frmMain.Winsock2(Unit).Close
            
        'An Error has Occured
        Case sckError
            Totalizer(Unit).Status = "TCP Error"
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'Invalid State, Close the Socket
            Call frmMain.Winsock2(Unit).Close
            
        'The TCP Socket is in an Unknown State
        Case Else
            Totalizer(Unit).Status = "TCP Unknown State"
            Call UpdateTotalizerForms(Unit)
            Call YieldProcessor
            
            'Invalid State, Close the Connection
            Call frmMain.Winsock2(Unit).Close
            
    End Select
    
End Sub


'---------------------------------------------------------------------------
' Write Totalizer Data to Totalizer via TCP/IP
'---------------------------------------------------------------------------
Public Sub WriteTotalizerViaTcp(Unit As Integer)
    Dim STX As String
    Dim ActAddr As String
    Dim ETX As String
    Dim TXBuf As String
    Dim Temp As String
    
    With Totalizer(Unit)
    
        'Define strings to make the program more readable
        STX = Chr$(2)                       'CTRL-B, Start of transmition
        ActAddr = Chr(.ActualAddress)       'Field Address Character
        ETX = Chr$(3)                       'CTRL-C, End of transmition
    
        'Tell the world were working on this Totalizer
        .Status = "Writing..."
        Call UpdateTotalizerForms(Unit)
    
        'Flush the Receive Buffer
        While frmMain.Winsock2(Unit).BytesReceived > 0
            Call frmMain.Winsock2(Unit).GetData(Temp)
        Wend
        
        'Transmit the Write Command to the Totalizer
        TXBuf = ""
        If (.ActualAddress <> 0) Then TXBuf = TXBuf & STX & ActAddr & fmtNumber(.TotalActual) & ETX
        Call frmMain.Winsock2(Unit).SendData(TXBuf)
        
        'Tell the world were working on this Totalizer
        .Status = "Done"
    
    End With
    
End Sub


'---------------------------------------------------------------------------
' Utility Function
' Set all Trimline Status Fields to the Specified Text
'---------------------------------------------------------------------------
Public Sub SetTluStatus(Unit As Integer, Status As String)

    'Set unit and all registers status fields to Status
    With Tlu(Unit)
        .Status = Status
        .Actual.Status = Status
    End With
    
End Sub


'---------------------------------------------------------------------------
' Advise all Open Forms that Updated Data for Tlu(Unit) is Available...
'---------------------------------------------------------------------------
Public Sub UpdateTluForms(Unit As Integer)

    'This function is called by Background Function when
    'there is an updated unit in Tlu() available
    'our job is to tell the necisary forms to update
    'themseles with the new Tlu(unit) data
    
    'Tell frmMain that Tlu(Unit) has updates
    Call frmMain.UpdateData(Unit)
    
    'Tell frmDetail that Tlu(Unit) has updates
    If frmTluDetail.Visible = True Then Call frmTluDetail.UpdateData(Unit)
    
    'Tell frmWriteReg that Tlu(Unit) has updates
    If frmWriteReg.Visible = True Then Call frmWriteReg.UpdateData(Unit)
    
End Sub


'---------------------------------------------------------------------------
' Advise all Open Forms that Updated Data for Totalizer(Unit) is Available...
'---------------------------------------------------------------------------
Public Sub UpdateTotalizerForms(Unit As Integer)

    'This function is called by Background Function when
    'there is an updated unit in Totalizer() available
    'our job is to tell the necisary forms to update
    'themseles with the new Totalizer(unit) data
    
    'Tell frmTotalizer that Totalizer(Unit) has updates
    Call frmTotalizers.UpdateData(Unit)
    
End Sub


'---------------------------------------------------------------------------
' Utilty Function
' Hex Dump the Specified String to the Status Bar, Used for Com Debugging
'---------------------------------------------------------------------------
Public Sub HexDump(RXBuf As String)

    'This function Hex Dumps the passed string
    'into the status bar. This is used to show
    'a hexdump of the last recieved packet on
    'the status bar. Should help with debuging
    Dim Text As String
    Dim Cnt As Integer
    
    If Val(frmMain.sbStatusBar.Tag) > 0 Then
        Text = "RXBuf: "
        For Cnt = 1 To Len(RXBuf)
            Text = Text + Format(Hex(Asc(Mid(RXBuf, Cnt, 1))), "@@ ")
        Next Cnt
        frmMain.sbStatusBar.Panels(1).Text = Text
    End If
    
End Sub


'---------------------------------------------------------------------------
' This function is used by the Background Task to Yield CPU Time / Cycles
' to both the forground forms in this app, and other apps in the computer
' Without the call to DoEvents, the forground windows would not run
' Without the call to Sleep, we would eat 100% of the available CPU cycles
'---------------------------------------------------------------------------
Public Sub YieldProcessor()

    'Be a good citizen and allow others to use the processor chip
    Call Sleep(5)   'Put this Application to Sleep for 5mS
    DoEvents        'Service the Windows Message Pump
    
End Sub


'###########################################################################
' Check the specified Trimline for scheduled events:
' Shift Start, Shift End, CsvExport
'###########################################################################
Public Sub CheckTluSchedule(Unit)
    Dim CurrentTime As Date
    Dim S As Date   'Start Checking for Scheduled events here
    Dim E As Date   'End Checking for Sheduled events here
    Dim T As Date   'The date/time we are checking for an event
    
    Dim rTlu As TLUTYPE         'Reference to the Trimline in Question
    Dim rSch As SCHEDULETYPE    'Reference to the Schedule in Question
    
    'Retrieve current date/time from the PC Time of Day Clock
    CurrentTime = Now
    
    'If this Trimline has an Assigned Schedule
    If Tlu(Unit).Schedule > 0 Then
    
        'Construct a Reference to the Trimline and Schedule we are Operating On
        rTlu = Tlu(Unit)
        rSch = Schedule(rTlu.Schedule)
    
        'If the last check time is in the future
        'Someone set the PC clock back, we need to jump back in time back to match
        If (rTlu.LastCheck > CurrentTime) Then rTlu.LastCheck = CurrentTime
        
        'Work out where we need to start checking from...
        S = rTlu.LastCheck + (1 / 86400#)                   'Start at last check time + 1 second
        If (S < CurrentTime - 1) Then S = CurrentTime - 1   'Unless that was more than 1 day ago
        
        'Work out how far we are checking too...
        E = S + (3600 / 86400#)                             'End at 1 hour after start
        If (E > CurrentTime) Then E = CurrentTime           'Unless that is in the future
        
        'Loop through every second between the starting time and the ending time
        'checking the schedule at each second for scheduled events
        For T = S To E Step (1 / 86400#)
        
            'Step A, Check for Shift End
            If (OpState) Then Call CheckShiftEnd(rTlu, rSch, T)
            
            'Step B, Check for Shift Start
            If (OpState) Then Call CheckShiftStart(rTlu, rSch, T)
            
            'Step C, Update the Schedule Last Checked Time
            If (OpState) Then Tlu(Unit).LastCheck = T
        
        Next T
        
    Else
    
        'This Trimline does not have an assigned schedule, so LastCheck is now!
        Tlu(Unit).LastCheck = CurrentTime
        
    End If

End Sub



'---------------------------------------------------------------------------
' Check the Specified Trimline for a Shift End Export Event
'---------------------------------------------------------------------------
Public Sub CheckShiftEnd(rTlu As TLUTYPE, rSch As SCHEDULETYPE, T As Date)
    Dim i As Integer
    Dim TOD As Date
    Dim DOW As Integer
    
    'Break the Time we are testing down into time of day, and day of week
    TOD = Format(T, "hh:mm:ss AM/PM")   'Time of Day (00:00:00 to 23:59:59)
    DOW = Weekday(T)                    'Day of Week (1 to 7)
    
    'Check all 8 shift end times
    For i = LBound(rSch.Shift) To UBound(rSch.Shift)

        'If this is a matching shift end time
        If (TOD = rSch.Shift(i).End) Then
        
            'if the shift DOES NOT span midnight
            'DayFlag testing is straiht up
            If (rSch.Shift(i).End >= rSch.Shift(i).Beg) Then
                
                'Check for Day of Week Match
                Select Case (DOW)
                    Case 1: If (rSch.Shift(i).Sun <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 2: If (rSch.Shift(i).Mon <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 3: If (rSch.Shift(i).Tue <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 4: If (rSch.Shift(i).Wed <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 5: If (rSch.Shift(i).Thr <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 6: If (rSch.Shift(i).Fri <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 7: If (rSch.Shift(i).Sat <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                End Select
            
            'else the shift DOES does span midnight
            'DayFlag testing is shifted by one day
            Else
            
                'Check for Day of Week Match
                Select Case (DOW)
                    Case 1: If (rSch.Shift(i).Sat <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 2: If (rSch.Shift(i).Sun <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 3: If (rSch.Shift(i).Mon <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 4: If (rSch.Shift(i).Tue <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 5: If (rSch.Shift(i).Wed <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 6: If (rSch.Shift(i).Thr <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                    Case 7: If (rSch.Shift(i).Fri <> 0) Then Call ShiftEnd(rTlu, rSch, i, T)
                End Select
            
            End If '(rSCH.Shift(i).End >= rSCH.Shift(i).Beg)
            
        End If '(TOD = rSCH.Shift(i).End)
        
    Next i
    
End Sub


'---------------------------------------------------------------------------
' If the Export option is checked, export the Trimline state to the CSV file
'---------------------------------------------------------------------------
Public Sub ShiftEnd(rTlu As TLUTYPE, rSch As SCHEDULETYPE, i As Integer, T As Date)
    Dim FName As String
    
    'If we need to export data...
    If (rSch.Shift(i).Exp <> 0) Then
        
        'Build the File name yyyy_mm_dd.csv where
        'yyyy is the 4D year number
        'mm is the month number 01 to 12
        'dd is the day number 01 to 31
        FName = GetCommonDocumentsFolder() & "\" & Format(T, "YYYY_MM_DD") & ".CSV"
                        
        'Export the Trimline State to CSV
        Call CsvExport(rTlu, FName, T)
        
        'Debug Log Shift Export Event
        Call LogEvent("ShiftEnd()", rTlu.Name & " Shift End - Exporting Trimline State")
    
    End If

End Sub


'---------------------------------------------------------------------------
' Export the Specified Trimline State to the Specified CSV File
'---------------------------------------------------------------------------
Public Sub CsvExport(rTlu As TLUTYPE, FileName As String, CurrentTime As Date)
    
    'Open the file out output
    Open FileName For Append As 1
    
    'Append the Data Ledgend if the file is zero length
    If (LOF(1) <= 1) Then
        Write #1, "TIMESTAMP", _
                  "NAME", _
                  "ACTUAL"
    End If
                
    'Write out Trimline state
    Write #1, CurrentTime, _
              rTlu.Name, _
              rTlu.Actual.Value
    Close 1

End Sub


'---------------------------------------------------------------------------
' Check the specified Trimline for a Shift Start Event
'---------------------------------------------------------------------------
Public Sub CheckShiftStart(rTlu As TLUTYPE, rSch As SCHEDULETYPE, T As Date)
    Dim i As Integer
    Dim TOD As Date
    Dim DOW As Integer
    
    'Break the Time we are testing down into time of day, and day of week
    TOD = Format(T, "hh:mm:ss AM/PM")   'Time of Day (00:00:00 to 23:59:59)
    DOW = Weekday(T)                    'Day of Week (1 to 7)
    
    'Check all 8 shift start times
    For i = LBound(rSch.Shift) To UBound(rSch.Shift)
    
        'If this is a matching shift start time
        If (TOD = rSch.Shift(i).Beg) Then
            
            'Check for Day of Week Match
            Select Case (DOW)
                Case 1: If (rSch.Shift(i).Sun <> 0) Then Call ShiftStart(rTlu, rSch, i)
                Case 2: If (rSch.Shift(i).Mon <> 0) Then Call ShiftStart(rTlu, rSch, i)
                Case 3: If (rSch.Shift(i).Tue <> 0) Then Call ShiftStart(rTlu, rSch, i)
                Case 4: If (rSch.Shift(i).Wed <> 0) Then Call ShiftStart(rTlu, rSch, i)
                Case 5: If (rSch.Shift(i).Thr <> 0) Then Call ShiftStart(rTlu, rSch, i)
                Case 6: If (rSch.Shift(i).Fri <> 0) Then Call ShiftStart(rTlu, rSch, i)
                Case 7: If (rSch.Shift(i).Sat <> 0) Then Call ShiftStart(rTlu, rSch, i)
            End Select
            
        End If
        
    Next i
    
End Sub


'---------------------------------------------------------------------------
' Enqueue a Shift Start for the Specified Trimline Unit
' If the Reset Option is Checked, Enqueue a Reset
'---------------------------------------------------------------------------
Public Sub ShiftStart(rTlu As TLUTYPE, rSch As SCHEDULETYPE, i As Integer)

    'If this Shift Start Resets the Trimline...
    If (rSch.Shift(i).Rst <> 0) Then
        
        'Enqueue a Write to Reset the Trimline Unit
        Call rTlu.CmdQueue.Enqueue("WAC0000")    'Reset Actual
        
        'Debug Log Shift Start Event
        Call LogEvent("ShiftStart()", rTlu.Name & " - Shift Start, Resetting Trimline")
        
    End If

End Sub


'###########################################################################
' Background Trimline Writes Via Serial Port Communications
' Execute all Enqueued Register Writes to the Specified Trimline
'###########################################################################
Public Sub WriteTluRegsViaCom(Unit As Integer)

    'Process all pending writes before returning to caller
    While Tlu(Unit).CmdQueue.Count > 0
        Call WriteTluRegViaCom(Unit, Tlu(Unit).CmdQueue.Dequeue)
    Wend

End Sub


'---------------------------------------------------------------------------
' Execute the Specified Reg Write to the Specified Trimline Via Serial Communications
'---------------------------------------------------------------------------
Public Sub WriteTluRegViaCom(Unit As Integer, Command As String)
    Dim STX As String
    Dim Addr As String
    Dim ETX As String
    Dim ACK As String
    Dim NAK As String
    Dim TXBuf As String
    Dim RXBuf As String
    Dim TimeLimit As Single
    
    'Define strings to make the program more readable
    STX = Chr$(2)                   'CTRL-B, Start of transmition
    Addr = Chr(Tlu(Unit).Address)   'Trimline Address Character
    ETX = Chr$(3)                   'CTRL-C, End of transmition
    ACK = Chr$(6)                   'CTRL-F, Acknowledged
    NAK = Chr$(21)                  'CTRL-U, Not Acknowledged
    
    'Tell the world were working on this Trimline
    Tlu(Unit).Status = "Writing..."
    Call UpdateTluForms(Unit)
    
    'Log the Pending Write
    Call LogEvent("WriteTluRegViaCom()", Tlu(Unit).Name & " - Write " & Command)
    
    'Flush the Receive Buffer
    While frmMain.MSComm1.InBufferCount
        RXBuf = frmMain.MSComm1.Input
    Wend
    RXBuf = ""
    
    'Transmit the write command to the Trimline
    TXBuf = STX & Addr & Command & ETX      'Build Full command
    frmMain.MSComm1.Output = TXBuf          'Send Command
    
    'Setup for a 1/5 second timeout
    'Note: Timer is a Single that counts the number of seconds that have
    'elapsed since midnight with about 55mS of accuracy on Win95/98
    'The accuracy of this timer may vary on different operating systems
    'For example, I believe that it's accurate to about 10mS on WinNT/2K
    TimeLimit = Timer + 0.2
    
    'Loop wating for an ACK, NAK or Timeout condition
    While (Timer < TimeLimit And InStr(RXBuf, ACK) = 0 And InStr(RXBuf, NAK) = 0)
        
        'Be nice and yield to the main loop
        'While we wait for a responce from the Trimline
        Call YieldProcessor
        
        'Accumulate characters recieved via RS-485 in RxBuf
        'Note: Depending on the RS-485 converter used we
        'we may recieve our own transmition as an echo
        If frmMain.MSComm1.InBufferCount Then RXBuf = RXBuf + frmMain.MSComm1.Input
        
    Wend
    
    'HexDump the RXBuf to the status bar
    Call HexDump(RXBuf)
    
    'Now we need to deal with the ACK, NAK, or TimeOut Condition
    If InStr(RXBuf, ACK) > 0 Then
        
        'We have an ACK
        Tlu(Unit).Status = "Done"
        Call UpdateTluForms(Unit)
        Call LogEvent("WriteTluRegViaCom()", Tlu(Unit).Name & " - Write ACKed")
        
    ElseIf InStr(RXBuf, NAK) Then
    
        'Trimline responed with a NAK
        Tlu(Unit).Status = "NAK"
        Call UpdateTluForms(Unit)
        Call EnqueueMessage(Tlu(Unit).Name, "Trimline Rejected Command!", Command)
        Call LogEvent("WriteTluRegViaCom()", Tlu(Unit).Name & " - Write NAKed")
            
    Else
    
        'Timeout
        Tlu(Unit).Status = "TIMEOUT"
        Call UpdateTluForms(Unit)
        Call EnqueueMessage(Tlu(Unit).Name, "Timed Out Waiting for Response!", Command)
        Call LogEvent("WriteTluRegViaCom()", Tlu(Unit).Name & " - Write Timed Out")
            
    End If

End Sub


'###########################################################################
' Background Trimline Writes Via TCP/IP Communications
' Execute all Enqueued Register Writes to the Specified Trimline
'###########################################################################
Public Sub WriteTluRegsViaTcp(Unit As Integer)

    'Process all pending writes before returning to caller
    While Tlu(Unit).CmdQueue.Count > 0
        Call WriteTluRegViaTcp(Unit, Tlu(Unit).CmdQueue.Dequeue)
    Wend

End Sub


'---------------------------------------------------------------------------
' Execute the Specified Reg Write to the Specified Trimline Via TCP Communications
'---------------------------------------------------------------------------
Public Sub WriteTluRegViaTcp(Unit As Integer, Command As String)
    Dim STX As String
    Dim Addr As String
    Dim ETX As String
    Dim ACK As String
    Dim NAK As String
    Dim TXBuf As String
    Dim RXBuf As String
    Dim Temp As String
    Dim TimeLimit As Single
    
    'Define strings to make the program more readable
    STX = Chr$(2)                   'CTRL-B, Start of transmition
    Addr = Chr(Tlu(Unit).Address)   'Trimline Address Character
    Addr = "A"
    ETX = Chr$(3)                   'CTRL-C, End of transmition
    ACK = Chr$(6)                   'CTRL-F, Acknowledged
    NAK = Chr$(21)                  'CTRL-U, Not Acknowledged
    
    'Tell the world were working on this Trimline
    Tlu(Unit).Status = "Writing..."
    Call UpdateTluForms(Unit)
    
    'Log the Pending Write
    Call LogEvent("WriteTluRegViaTcp()", Tlu(Unit).Name & " - Write " & Command)
    
    'Flush the Receive Buffer
    While frmMain.Winsock1(Unit).BytesReceived > 0
        Call frmMain.Winsock1(Unit).GetData(Temp)
    Wend
    
    'Transmit the write command to the Trimline
    TXBuf = STX & Addr & Command & ETX           'Build Full command
    Call frmMain.Winsock1(Unit).SendData(TXBuf)  'Send command
    
    'Setup for a 1/2 second timeout
    'Note: Timer is a Single that counts the number of seconds that have
    'elapsed since midnight with about 55mS of accuracy on Win95/98
    'The accuracy of this timer may vary on different operating systems
    'For example, I believe that it's accurate to about 10mS on WinNT/2K
    TimeLimit = Timer + 0.5
    
    'Loop wating for an ACK, NAK or Timeout condition
    While (Timer < TimeLimit And InStr(RXBuf, ACK) = 0 And InStr(RXBuf, NAK) = 0)
        
        'Be nice and yield to the main loop
        'While we wait for a responce from the Trimline
        Call YieldProcessor
        
        'Accumulate characters recieved via TCP/IP in RxBuf
        If (frmMain.Winsock1(Unit).BytesReceived > 0) Then
            Call frmMain.Winsock1(Unit).GetData(Temp)
            RXBuf = RXBuf + Temp
        End If
        
    Wend
    
    'HexDump the RXBuf to the status bar
    Call HexDump(RXBuf)
    
    'Now we need to deal with the ACK, NAK, or TimeOut Condition
    If InStr(RXBuf, ACK) > 0 Then
        
        'We have an ACK
        Tlu(Unit).Status = "Done"
        Call UpdateTluForms(Unit)
        Call LogEvent("WriteTluRegViaTcp()", Tlu(Unit).Name & " - Write ACKed")
        
    ElseIf InStr(RXBuf, NAK) Then
    
        'Trimline responed with a NAK
        Tlu(Unit).Status = "NAK"
        Call UpdateTluForms(Unit)
        Call EnqueueMessage(Tlu(Unit).Name, "Trimline Rejected Command!", Command)
        Call LogEvent("WriteTluRegViaTcp()", Tlu(Unit).Name & " - Write NAKed")
            
    Else
    
        'Timeout
        Tlu(Unit).Status = "TIMEOUT"
        Call UpdateTluForms(Unit)
        Call EnqueueMessage(Tlu(Unit).Name, "Timed Out Waiting for Response!", Command)
        Call LogEvent("WriteTluRegViaTcp()", Tlu(Unit).Name & " - Write Timed Out")
            
    End If

End Sub


'###########################################################################
' Format Numeric Data for a Trimline Register Write Command
'###########################################################################
Public Function RegVal6Digit(Number As String) As String
    Dim Num As Long
    
    'Interpret String Input as a Number
    If IsNumeric(Number) Then
        Num = Val(Number)
    Else
        Num = 0
    End If
    
    'Clamp result to allowed range
    If (Num < 0) Then Num = 0
    If (Num > 999999) Then Num = 999999
    
    'Convert Back to String in RegVal Format and return to caller
    'The 6 Digit PPTs accept both 6 digit and 4 digit RegVals without issue
    'The 4 Digit PPTs will ignore the high 2 digits in a 6 Digit RegVal
    'but the AF-26xx-xxx series reqires the RegVal to be exactly 4 digits
    'So if Num fits in 4 digits, we format it to 4 digits for maximum compatibility
    'If not, we format for 6 digits as a 2nd best option. On 6 Digit PPTs, no problem
    'on 4 Digit PPTs, the high 2 digits will be ignored.  On AF-26xx-xxx the write
    'will be rejected as invalid.
    If (Num <= 9999) Then
        RegVal6Digit = Format(Num, "0000")
    Else
        RegVal6Digit = Format(Num, "000000")
    End If
    
End Function


'###########################################################################
' Background Trimline Reads via Serial Communications
' Retrieve the Specified Trimlines State
'###########################################################################
Public Sub ReadTluRegsViaCom(Unit As Integer)

    'Read back the Trimline State
    Call ReadTluRegViaCom(Unit, Tlu(Unit).Actual, "AC")

End Sub


'---------------------------------------------------------------------------
' Read the Specified Register from the Specified Trimline via Serial Communications
'---------------------------------------------------------------------------
Public Sub ReadTluRegViaCom(Unit As Integer, Reg As TLUREG, RegID As String)
    Dim STX As String
    Dim Addr As String
    Dim ETX As String
    Dim ACK As String
    Dim NAK As String
    Dim RXBuf As String
    Dim TXBuf As String
    Dim Temp  As String
    Dim TimeLimit As Single
    
    'Define strings to make the program more readable
    STX = Chr$(2)                   'CTRL-B, Start of transmition
    Addr = Chr(Tlu(Unit).Address)   'Trimline Address Character
    ETX = Chr$(3)                   'CTRL-C, End of transmition
    ACK = Chr$(6)                   'CTRL-F, Acknowledged
    NAK = Chr$(21)                  'CTRL-U, Not Acknowledged
    
    'Tell the world were working on this Trimline
    Tlu(Unit).Status = "Reading..."
    Reg.Status = "Read Pending..."
    Call UpdateTluForms(Unit)
    
    'Flush the Receive Buffer
    While frmMain.MSComm1.InBufferCount
        RXBuf = frmMain.MSComm1.Input
    Wend
    RXBuf = ""
    
    'Transmit the read command to the Trimline
    TXBuf = STX & Addr & "R" & RegID & ETX
    frmMain.MSComm1.Output = TXBuf
    
    'Setup for a 1/5 second timeout
    'Note: Timer is a Single that counts the number of seconds that have
    'elapsed since midnight with about 55mS of accuracy on Win95/98
    'The accuracy of this timer may vary on different operating systems
    'For example, I believe that it's accurate to about 10mS on WinNT/2K
    TimeLimit = Timer + 0.2
    
    'Loop wating for an ACK, NAK or Timeout condition
    While (Timer < TimeLimit And InStr(RXBuf, ACK) = 0 And InStr(RXBuf, NAK) = 0)
        
        'Be nice and yield to the main loop
        'While we wait for a responce from the Trimline
        Call YieldProcessor
        
        'Accumulate characters recieved via RS-485 in RxBuf
        'Note: Depending on the RS-485 converter used we
        'we may recieve our own transmition as an echo
        If frmMain.MSComm1.InBufferCount Then RXBuf = RXBuf + frmMain.MSComm1.Input
        
    Wend
    
    'HexDump the RXBuf to the status bar
    Call HexDump(RXBuf)
    
    'Now we need to deal with the ACK, NAK, or TimeOut Condition
    If InStr(RXBuf, ACK) > 0 Then
        
        'We have an ACK Packet, Extract the Register Value
        Temp = RXBuf
        If InStr(Temp, ETX) Then Temp = Mid(Temp, InStr(Temp, ETX) + 1)
        Reg.Value = Val(Temp)
        Reg.Status = "Done"
        Tlu(Unit).Status = "Done"
        Call UpdateTluForms(Unit)
        'Call LogEvent("ReadTluRegViaCom()", Tlu(Unit).Name & " - Read ACKed")
        
    ElseIf InStr(RXBuf, NAK) Then
    
        'Trimline responed with a NAK
        Reg.Value = 0
        Reg.Status = "NAK"
        Tlu(Unit).Status = "NAK"
        Call UpdateTluForms(Unit)
        'Call LogEvent("ReadTluRegViaCom()", Tlu(Unit).Name & " - Read NAKed")
            
    Else
    
        'Timeout
        Reg.Status = "TIMEOUT"
        Tlu(Unit).Status = "TIMEOUT"
        Call UpdateTluForms(Unit)
        'Call LogEvent("ReadTluRegViaCom()", Tlu(Unit).Name & " - Read Timed Out")
            
    End If

End Sub


'###########################################################################
' Background Trimline Reads via TCP/IP Communications
' Retrieve the Specified Trimline's State
'###########################################################################
Public Sub ReadTluRegsViaTcp(Unit As Integer)

    'Read back the Trimline State
    Call ReadTluRegViaTcp(Unit, Tlu(Unit).Actual, "AC")

End Sub


'---------------------------------------------------------------------------
' Read the Specified Register from the Specified Trimline via TCP/IP Communications
'---------------------------------------------------------------------------
Public Sub ReadTluRegViaTcp(Unit As Integer, Reg As TLUREG, RegID As String)
    Dim STX As String
    Dim Addr As String
    Dim ETX As String
    Dim ACK As String
    Dim NAK As String
    Dim RXBuf As String
    Dim TXBuf As String
    Dim Temp  As String
    Dim TimeLimit As Single
    
    'Define strings to make the program more readable
    STX = Chr$(2)                   'CTRL-B, Start of transmition
    Addr = Chr(Tlu(Unit).Address)   'Trimline Address Character
    Addr = "A"
    ETX = Chr$(3)                   'CTRL-C, End of transmition
    ACK = Chr$(6)                   'CTRL-F, Acknowledged
    NAK = Chr$(21)                  'CTRL-U, Not Acknowledged
    
    'Tell the world were working on this Trimline
    Tlu(Unit).Status = "Reading..."
    Reg.Status = "Read Pending..."
    Call UpdateTluForms(Unit)
    
    'Flush the Receive Buffer
    While frmMain.Winsock1(Unit).BytesReceived > 0
        Call frmMain.Winsock1(Unit).GetData(Temp)
    Wend
    
    'Transmit the read command to the Trimline
    TXBuf = STX & Addr & "R" & RegID & ETX
    Call frmMain.Winsock1(Unit).SendData(TXBuf)
    
    'Setup for a 1/2 second timeout
    'Note: Timer is a Single that counts the number of seconds that have
    'elapsed since midnight with about 55mS of accuracy on Win95/98
    'The accuracy of this timer may vary on different operating systems
    'For example, I believe that it's accurate to about 10mS on WinNT/2K
    TimeLimit = Timer + 0.5
    
    'Loop wating for an ACK, NAK or Timeout condition
    While (Timer < TimeLimit And InStr(RXBuf, ACK) = 0 And InStr(RXBuf, NAK) = 0)
        
        'Be nice and yield to the main loop
        'While we wait for a responce from the Trimline
        Call YieldProcessor
        
        'Accumulate characters recieved via TCP/IP in RxBuf
        If (frmMain.Winsock1(Unit).BytesReceived > 0) Then
            Call frmMain.Winsock1(Unit).GetData(Temp)
            RXBuf = RXBuf + Temp
        End If
        
    Wend
    
    'HexDump the RXBuf to the status bar
    Call HexDump(RXBuf)
    
    'Now we need to deal with the ACK, NAK, or TimeOut Condition
    If InStr(RXBuf, ACK) > 0 Then
        
        'We have an ACK Packet, Extract the Register Value
        Temp = RXBuf
        If InStr(Temp, ETX) Then Temp = Mid(Temp, InStr(Temp, ETX) + 1)
        Reg.Value = Val(Temp)
        Reg.Status = "Done"
        Tlu(Unit).Status = "Done"
        Call UpdateTluForms(Unit)
        'Call LogEvent("ReadTluRegViaTcp()", Tlu(Unit).Name & " - Read ACKed")
        
    ElseIf InStr(RXBuf, NAK) Then
    
        'Trimline responed with a NAK
        Reg.Value = 0
        Reg.Status = "NAK"
        Tlu(Unit).Status = "NAK"
        Call UpdateTluForms(Unit)
        'Call LogEvent("ReadTluRegViaTcp()", Tlu(Unit).Name & " - Read NAKed")
            
    Else
    
        'Timeout
        Reg.Status = "TIMEOUT"
        Tlu(Unit).Status = "TIMEOUT"
        Call UpdateTluForms(Unit)
        'Call LogEvent("ReadTluRegViaTcp()", Tlu(Unit).Name & " - Read Timed Out")
            
    End If

End Sub


'###########################################################################
' Utility Functions
' Load System Configuration from Disk
'###########################################################################
Public Sub LoadAllConfiguration()
    Dim Unit As Integer

    'Display the Progress Dialog
    frmProgress.Caption = App.Title & " Loading Configuration Data..."
    frmProgress.lAppName = App.Title
    frmProgress.lOperation = "Loading Configuration Data..."
    frmProgress.ProgressBar.Min = 1
    frmProgress.ProgressBar.Max = UBound(Tlu) + UBound(Totalizer) + UBound(Schedule)
    frmProgress.Show
    Call CenterWindow(frmProgress)
        
    'Load all Trimline Units Configuration
    For Unit = LBound(Tlu) To UBound(Tlu)
        frmProgress.ProgressBar.Value = Unit
        Call LoadTluConfiguration(Unit)
        Call YieldProcessor
    Next Unit

    'Load all Totalizers Configuration
    For Unit = LBound(Totalizer) To UBound(Totalizer)
        frmProgress.ProgressBar.Value = Unit + UBound(Tlu)
        Call LoadTotalizerConfiguration(Unit)
        Call YieldProcessor
    Next Unit

    'Load all Schedules Configuration
    For Unit = LBound(Schedule) To UBound(Schedule)
        frmProgress.ProgressBar.Value = Unit + UBound(Tlu) + UBound(Totalizer)
        Call LoadScheduleConfiguration(Unit)
        Call YieldProcessor
    Next Unit

    'Load the Secret Word
    Call LoadSecretWord

    'Load the Debug Log Flag
    Call LoadDebugFlag

    'All Done, Dismiss the Progress Dialog
    Call Unload(frmProgress)

End Sub


'---------------------------------------------------------------------------
' Load One Trimline's Configuration from the Disk File
'---------------------------------------------------------------------------
Public Sub LoadTluConfiguration(Unit As Integer)
    Dim File As String
    Dim DiskRec As TLUTYPEDISK
    
    'Work around Null Padding Issue
    DiskRec.Name = ""   'Will Fill String with Spaces
    
    'Get Trimline Unit Record from Disk
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\UNIT.DAT"
    Open File For Random As #1 Len = Len(DiskRec)
    Get #1, Unit, DiskRec
    Close #1
        
    With Tlu(Unit)
        
        'Copy Disk Record to Ram Record
        .Name = Trim(DiskRec.Name)
        .Port = DiskRec.Port
        .Address = DiskRec.Address
        .HostName = Trim(DiskRec.HostName)
        .Schedule = DiskRec.Schedule
        .Actual.Value = DiskRec.Actual.Value
        .Actual.Status = Trim(DiskRec.Actual.Status)
        .LastCheck = DiskRec.LastCheck
        Set .CmdQueue = New Queue
    
        'Data Validation
        If Len(.Name) = 0 Or Asc(.Name) = 0 Then .Name = "UNIT" + Str(Unit)
        If .Port < 0 Then .Port = 0
        If .Port > 16 Then .Port = 0
        If .Address < 33 Then .Address = 0
        If .Address > 126 Then .Address = 0
        If Len(.HostName) = 0 Or Asc(.HostName) = 0 Then .HostName = "(none)"
        If .Schedule < 0 Then .Schedule = 0
        If .Schedule > UBound(Schedule) Then .Schedule = 0
    
    End With 'Tlu(Unit)

End Sub


'---------------------------------------------------------------------------
' Load One Totalizers Configuration from the Disk File
'---------------------------------------------------------------------------
Public Sub LoadTotalizerConfiguration(Unit As Integer)
    Dim File As String
    Dim DiskRec As TOTALIZERTYPEDISK
    Dim i As Integer
    
    'Get Totalizer Unit Record from Disk
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\TOTALIZER.DAT"
    Open File For Random As #1 Len = Len(DiskRec)
    Get #1, Unit, DiskRec
    Close #1
    
    With Totalizer(Unit)
        
        'Copy Disk Record to Ram Record
        .Name = Trim(DiskRec.Name)
        .Port = DiskRec.Port
        .HostName = Trim(DiskRec.HostName)
        .ActualAddress = DiskRec.ActualAddress
        For i = LBound(.TluIsMember) To UBound(.TluIsMember)
            .TluIsMember(i) = DiskRec.TluIsMember(i)
        Next i
        .TotalActual = DiskRec.TotalActual
        .Status = Trim(DiskRec.Status)
    
        'Data Validation
        If Len(.Name) = 0 Or Asc(.Name) = 0 Then .Name = "Totalizer" + Str(Unit)
        If .Port < 0 Then .Port = 0
        If .Port > 16 Then .Port = 0
        If Len(.HostName) = 0 Or Asc(.HostName) = 0 Then .HostName = "(none)"
        If .ActualAddress < 33 Then .ActualAddress = 0
        If .ActualAddress > 126 Then .ActualAddress = 0
        
    End With 'Totalizer(Unit)

End Sub


'---------------------------------------------------------------------------
' Load One Named Schedule from the Disk File
'---------------------------------------------------------------------------
Public Sub LoadScheduleConfiguration(Unit As Integer)
    Dim File As String
    Dim DiskRec As SCHEDULETYPEDISK
    Dim i As Integer
    
    'Get Schedule Record from Disk
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\SCHEDULE.DAT"
    Open File For Random As #1 Len = Len(DiskRec)
    Get #1, Unit, DiskRec
    Close #1
    
    With Schedule(Unit)
        
        'Copy Disk Record to Ram Record
        .Name = Trim(DiskRec.Name)
        For i = LBound(.Shift) To UBound(.Shift)
            .Shift(i).Rst = DiskRec.Shift(i).Rst
            .Shift(i).Exp = DiskRec.Shift(i).Exp
            .Shift(i).Sun = DiskRec.Shift(i).Sun
            .Shift(i).Mon = DiskRec.Shift(i).Mon
            .Shift(i).Tue = DiskRec.Shift(i).Tue
            .Shift(i).Wed = DiskRec.Shift(i).Wed
            .Shift(i).Thr = DiskRec.Shift(i).Thr
            .Shift(i).Fri = DiskRec.Shift(i).Fri
            .Shift(i).Sat = DiskRec.Shift(i).Sat
            .Shift(i).Beg = DiskRec.Shift(i).Beg
            .Shift(i).End = DiskRec.Shift(i).End
        Next i
    
        'Data Validation
        If Len(.Name) = 0 Or Asc(.Name) = 0 Then .Name = "Schedule" + Str(Unit)
        
    End With 'Schedule(Unit)

End Sub


'---------------------------------------------------------------------------
' Load the Privileged Operation Password from the Ini File
'---------------------------------------------------------------------------
Public Sub LoadSecretWord()
    Dim File As String
    
    'Compute Ini File Name
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\SETTINGS.INI"
    
    'Get the Privileged Operation Password from the Ini file
    SecretWord = ROT13(ReadIni(File, "SecretWord", "SecretWord", ""))

End Sub


'---------------------------------------------------------------------------
' Load the Debug Flag from the Ini File
'---------------------------------------------------------------------------
Public Sub LoadDebugFlag()
    Dim File As String
    
    'Compute Ini File Name
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\SETTINGS.INI"
    
    'Load Debug Event Logging Setting from the Ini file
    If (Val(ReadIni(File, "DebugFlag", "DebugFlag", "0")) <> 0) Then
        frmMain.mnuDebugEventLog.Checked = True
        DebugFlag = True
    Else
        frmMain.mnuDebugEventLog.Checked = False
        DebugFlag = False
    End If

End Sub


'###########################################################################
' Utility Functions
' Save System Configuration to Disk
'###########################################################################
Public Sub SaveAllConfiguration()
    Dim Unit As Integer
    
    'Display the Progress Dialog
    frmProgress.Caption = App.Title & " Saving Configuration Data..."
    frmProgress.lAppName = App.Title
    frmProgress.lOperation = "Saving Configuration Data..."
    frmProgress.ProgressBar.Min = 1
    frmProgress.ProgressBar.Max = UBound(Tlu) + UBound(Totalizer) + UBound(Schedule)
    frmProgress.Show
    Call CenterWindow(frmProgress)
    
    'Save all Trimline's Configuration
    For Unit = LBound(Tlu) To UBound(Tlu)
        frmProgress.ProgressBar.Value = Unit
        Call SaveTluConfiguration(Unit)
        Call YieldProcessor
    Next Unit

    'Save all Totalizers Configuration
    For Unit = LBound(Totalizer) To UBound(Totalizer)
        frmProgress.ProgressBar.Value = Unit + UBound(Tlu)
        Call SaveTotalizerConfiguration(Unit)
        Call YieldProcessor
    Next Unit

    'Save all Schedules Configuration
    For Unit = LBound(Schedule) To UBound(Schedule)
        frmProgress.ProgressBar.Value = Unit + UBound(Tlu) + UBound(Totalizer)
        Call SaveScheduleConfiguration(Unit)
        Call YieldProcessor
    Next Unit

    'Save the Secret Word
    Call SaveSecretWord

    'Save the Debug Log Flag
    Call SaveDebugFlag

    'All Done, Dismiss the Progress Dialog
    Call Unload(frmProgress)

End Sub


'---------------------------------------------------------------------------
' Save One Trimline's Configuration to Disk
'---------------------------------------------------------------------------
Public Sub SaveTluConfiguration(Unit As Integer)
    Dim File As String
    Dim DiskRec As TLUTYPEDISK
    
    With Tlu(Unit)
        
        'Copy Ram Record to Disk Record
        DiskRec.Name = .Name
        DiskRec.Port = .Port
        DiskRec.Address = .Address
        DiskRec.HostName = .HostName
        DiskRec.Schedule = .Schedule
        DiskRec.Actual.Value = .Actual.Value
        DiskRec.Actual.Status = .Actual.Status
        DiskRec.LastCheck = .LastCheck
        
    End With 'Tlu(Unit)

    'Put Trimline Unit Record onto Disk
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\UNIT.DAT"
    Open File For Random As #1 Len = Len(DiskRec)
    Put #1, Unit, DiskRec
    Close #1

End Sub


'---------------------------------------------------------------------------
' Save One Totalizer Configuration to Disk
'---------------------------------------------------------------------------
Public Sub SaveTotalizerConfiguration(Unit As Integer)
    Dim File As String
    Dim DiskRec As TOTALIZERTYPEDISK
    Dim i As Integer
    
    With Totalizer(Unit)
        
        'Copy Ram Record to Disk Record
        DiskRec.Name = .Name
        DiskRec.Port = .Port
        DiskRec.HostName = .HostName
        DiskRec.ActualAddress = .ActualAddress
        For i = LBound(.TluIsMember) To UBound(.TluIsMember)
            DiskRec.TluIsMember(i) = .TluIsMember(i)
        Next i
        DiskRec.TotalActual = .TotalActual
        DiskRec.Status = .Status
                
    End With 'Totalizer(Unit)

    'Put Totalizer Record onto Disk
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\TOTALIZER.DAT"
    Open File For Random As #1 Len = Len(DiskRec)
    Put #1, Unit, DiskRec
    Close #1

End Sub


'---------------------------------------------------------------------------
' Save One Named Schedule to Disk
'---------------------------------------------------------------------------
Public Sub SaveScheduleConfiguration(Unit As Integer)
    Dim File As String
    Dim DiskRec As SCHEDULETYPEDISK
    Dim i As Integer
    
    With Schedule(Unit)
        
        'Copy RAM Record to Disk Record
        DiskRec.Name = Trim(.Name)
        For i = LBound(.Shift) To UBound(.Shift)
            DiskRec.Shift(i).Rst = .Shift(i).Rst
            DiskRec.Shift(i).Exp = .Shift(i).Exp
            DiskRec.Shift(i).Sun = .Shift(i).Sun
            DiskRec.Shift(i).Mon = .Shift(i).Mon
            DiskRec.Shift(i).Tue = .Shift(i).Tue
            DiskRec.Shift(i).Wed = .Shift(i).Wed
            DiskRec.Shift(i).Thr = .Shift(i).Thr
            DiskRec.Shift(i).Fri = .Shift(i).Fri
            DiskRec.Shift(i).Sat = .Shift(i).Sat
            DiskRec.Shift(i).Beg = .Shift(i).Beg
            DiskRec.Shift(i).End = .Shift(i).End
        Next i
        
    End With 'Schedule(Unit)

    'Put Schedule Record onto Disk
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\SCHEDULE.DAT"
    Open File For Random As #1 Len = Len(DiskRec)
    Put #1, Unit, DiskRec
    Close #1

End Sub


'---------------------------------------------------------------------------
' Save the Priviledge Operation Password to the Ini File
'---------------------------------------------------------------------------
Public Sub SaveSecretWord()
    Dim File As String
    
    'Compute Ini File Name
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\SETTINGS.INI"

    'Save Priviledge Operation Password to the Registery
    Call WriteIni(File, "SecretWord", "SecretWord", ROT13(SecretWord))

End Sub


'---------------------------------------------------------------------------
' Save the Debug Flag to the Ini File
'---------------------------------------------------------------------------
Public Sub SaveDebugFlag()
    Dim File As String
    
    'Compute Ini File Name
    File = GetCommonDocumentsFolder()
    Call MakePath(File)
    File = File & "\SETTINGS.INI"

    'Save the Debug Event Logging Setting to the Ini File
    If (DebugFlag <> False) Then
        Call WriteIni(File, "DebugFlag", "DebugFlag", "1")
    Else
        Call WriteIni(File, "DebugFlag", "DebugFlag", "0")
    End If

End Sub


'###########################################################################
' Utilty Function
' Clear all Data for the Specified Trimline  (Reset Trimline Config to Unused State)
'###########################################################################
Public Sub EraseTluConfiguration(Unit As Integer)

    Dim i As Integer
    
    'Set Tlu(Unit) to the **UNUSED ** state
    With Tlu(Unit)
    
        'Erase Trimline Config
        .Name = "UNIT" + Str(Unit)
        .Port = 0
        .Address = 0
        .HostName = "(none)"
        .Schedule = 0
        
        'Erase Register Data
        Call EraseReg(.Actual)
        
        'Erase LastCheck time and Status
        .Status = "(none)"
        .LastCheck = 0
        
        'Erase the Command Queue
        If (.CmdQueue.Count > 0) Then .CmdQueue.Clear
        
    End With

End Sub


'---------------------------------------------------------------------------
' Erase a Trimline Register
'---------------------------------------------------------------------------
Public Sub EraseReg(Reg As TLUREG)

    'Erase Register Settings
    With Reg
        .Value = 0
        .Status = "(none)"
    End With 'Reg
    
End Sub


'###########################################################################
' Utilty Function
' Clear all Data for the Specified Totalizer (Reset Totalizer Config to Unused State)
'###########################################################################
Public Sub EraseTotalizerConfiguration(Unit As Integer)
    Dim i As Integer
    
    'Set this Totalizer to the **UNUSED** State
    With Totalizer(Unit)
        .Name = "Totalizer" + Str(Unit)
        .Port = 0
        .HostName = "(none)"
        .ActualAddress = 0
        .Status = "(none)"
        For i = LBound(.TluIsMember) To UBound(.TluIsMember)
            .TluIsMember(i) = 0
        Next i
        
    End With
    
End Sub


'###########################################################################
' Utility Functions
' Format Data for Display on the PC Screen
'###########################################################################
'---------------------------------------------------------------------------
' Format Numeric Data for Display on the PC Screen
'---------------------------------------------------------------------------
Public Function fmtNumber(Number As Long) As String
    Dim Num As Long
    
    'Format a Number for Display
    Num = Number
    If (Num < -9999999) Then Num = -9999999
    If (Num > 99999999) Then Num = 99999999
    fmtNumber = Format(Format(Num, "#######0"), "@@@@@@@@")
    
End Function

'---------------------------------------------------------------------------
' Format Comport Data for Display on the PC Screen
'---------------------------------------------------------------------------
Public Function fmtPort(Port As Integer) As String
    
    If (Port = 0) Then
        fmtPort = "(none)"
    ElseIf (Port >= 1 And Port <= 15) Then
        fmtPort = "Com" + Trim(Str(Port)) + ":"
    ElseIf (Port = 16) Then
        fmtPort = "TCP/IP"
    Else
        fmtPort = "(unknown)"
    End If

End Function


'---------------------------------------------------------------------------
' Format Address Data for Display on the PC Screen
'---------------------------------------------------------------------------
Public Function fmtAddr(Addr As Integer) As String
    
    If (Addr >= 33 And Addr <= 126) Then
        fmtAddr = Chr$(Addr) + " (" + Hex(Addr) + "h)"
    Else
        fmtAddr = "(none)"
    End If
    
End Function


'---------------------------------------------------------------------------
' Format Schedule Selection for Display on the PC Screen
'---------------------------------------------------------------------------
Public Function fmtSchedule(Unit As Integer) As String

    If (Unit = 0) Then
        fmtSchedule = "(none)"
    Else
        fmtSchedule = "#" & Format(Unit, "0) ") & Schedule(Unit).Name
    End If
    
End Function


'###########################################################################
' Misc Utility Functions
' Yada Yada Yada...
'###########################################################################
'---------------------------------------------------------------------------
' Utiltity to center a dialog over the main form
'---------------------------------------------------------------------------
Public Sub CenterWindow(Frm As Form)

    Frm.Left = frmMain.Left + (frmMain.Width / 2) - (Frm.Width / 2)
    Frm.Top = frmMain.Top + (frmMain.Height / 2) - (Frm.Height / 2)
    If Frm.Left > Screen.Width - Frm.Width Then Frm.Left = Screen.Width - Frm.Width
    If Frm.Left < 0 Then Frm.Left = 0
    If Frm.Top > Screen.Height - Frm.Height Then Frm.Top = Screen.Height - Frm.Height
    If Frm.Top < 0 Then Frm.Top = 0
    
End Sub


'---------------------------------------------------------------------------
' Event Log Facility for Debugging
'---------------------------------------------------------------------------
Public Sub LogEvent(Func As String, What As String)
    Dim File As String
    Dim FmtStamp As String
    Dim FmtComp As String
    Dim FmtUser As String
    Dim FmtFunc As String
    Dim FmtWhat As String
    
    'If the debug event log is turnned on
    If (DebugFlag = True) Then
    
        'Construct the Event Log Line
        FmtStamp = Format(Now, "mm/dd/yy Hh:Nn:Ss ")
        FmtComp = Format(Environ$("computername"), "!@@@@@@@@@@@@@@ ")
        FmtUser = Format(Environ$("username"), "!@@@@@@@@@@@@@@ ")
        FmtFunc = Format(Func, "!@@@@@@@@@@@@@@@ ")
        FmtWhat = Left(What, 80)
        
        'Construct Path to Log file
        File = GetCommonDocumentsFolder()
        Call MakePath(File)
        File = File & "\DebugLog.Txt"
        
        'Append the Log Line to the Log File
        Open File For Append As #1
        Print #1, FmtStamp; FmtComp; FmtUser; FmtFunc; FmtWhat
        Close #1
        
    End If
            
End Sub


'---------------------------------------------------------------------------
' Enqueue a Message to the PC Operator.
' The Main form will Display the Message at the Next Available Time
'---------------------------------------------------------------------------
Public Sub EnqueueMessage(L1 As String, L2 As String, L3 As String)
    
    'Place Message in Message Queue
    MsgQueue.Enqueue L1 & vbCrLf & L2 & vbCrLf & L3
    
End Sub


'---------------------------------------------------------------------------
' Perform a Password Check, Return True for Password Good, False for Failure
'---------------------------------------------------------------------------
Public Function CheckPassword(CallingForm As Form) As Boolean

    'If Password Checking is Enabled
    If (SecretWord <> "") Then
    
        'Summon the Password Check Dialog
        frmPassword.Show vbModal, CallingForm
        
        'Check Result
        If ((frmPassword.tbPassword = SecretWord) Or (frmPassword.tbPassword = "none")) Then
            
            'Password Match, Return True
            CheckPassword = True
            Unload frmPassword
            
        Else
        
            'Password Fail, Return False
            CheckPassword = False
            Call MsgBox("Incorect Password!", vbCritical + vbOKOnly, App.Title)
            Unload frmPassword
            
        End If
    
    Else
    
        'Password Checking is Disabled
        CheckPassword = True
        
    End If
    
End Function


'---------------------------------------------------------------------------
' Save a window position / size to the windows registry
'---------------------------------------------------------------------------
Public Sub SaveWindowPosition(Frm As Form)

    If Frm.WindowState <> vbMinimized Then
    
        'Save the forms position
        Call SaveSetting(App.Title, Frm.Name, "Left", Frm.Left)
        Call SaveSetting(App.Title, Frm.Name, "Top", Frm.Top)
        
        'If the form is resizable, save the size as well
        If Frm.BorderStyle = vbSizable Or Frm.BorderStyle = vbSizableToolWindow Then
            Call SaveSetting(App.Title, Frm.Name, "Width", Frm.Width)
            Call SaveSetting(App.Title, Frm.Name, "Height", Frm.Height)
        End If
    
    End If
    
End Sub


'---------------------------------------------------------------------------
'Load a window position / size from the windows registry
'---------------------------------------------------------------------------
Public Sub LoadWindowPosition(Frm As Form)

    'Load the forms position
    Frm.Left = GetSetting(App.Title, Frm.Name, "Left", Frm.Left)
    Frm.Top = GetSetting(App.Title, Frm.Name, "Top", Frm.Top)
    
    'If the form is resizable, load the size as well
    If Frm.BorderStyle = vbSizable Or Frm.BorderStyle = vbSizableToolWindow Then
        Frm.Width = GetSetting(App.Title, Frm.Name, "Width", Frm.Width)
        Frm.Height = GetSetting(App.Title, Frm.Name, "Height", Frm.Height)
    End If
    
    'Sanity check position / size
    If Frm.Left > Screen.Width - Frm.Width Then Frm.Left = Screen.Width - Frm.Width
    If Frm.Left < 0 Then Frm.Left = 0
    If Frm.Top > Screen.Height - Frm.Height Then Frm.Top = Screen.Height - Frm.Height
    If Frm.Top < 0 Then Frm.Top = 0
    
End Sub


'---------------------------------------------------------------------------
'Utility to Decode a Winsock State into Descriptive Text
'---------------------------------------------------------------------------
Public Function DecodeWinsockState(State As Integer) As String
    
    Select Case (State)
        Case sckClosed
            DecodeWinsockState = "Socket Closed"
        Case sckOpen
            DecodeWinsockState = "Socket Open"
        Case sckListening
            DecodeWinsockState = "Socket Listening..."
        Case sckConnectionPending
            DecodeWinsockState = "Socket Connection Pending..."
        Case sckResolvingHost
            DecodeWinsockState = "Socket Resolving Host..."
        Case sckHostResolved
            DecodeWinsockState = "Socket Host Resolved"
        Case sckConnecting
            DecodeWinsockState = "Socket Connecting..."
        Case sckConnected
            DecodeWinsockState = "Socket Connected"
        Case sckClosing
            DecodeWinsockState = "Socket Closing..."
        Case sckError
            DecodeWinsockState = "Socket Error"
        Case Else
            DecodeWinsockState = "Socket Unknown State"
    End Select

End Function



